/*
 * Copyright (C) 2009, Martin Robinson
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef DataObjectClutter_h
#define DataObjectClutter_h

#include "FileList.h"
#include "KURL.h"
#include "Range.h"
#include "RefPtrCairo.h"
#include <wtf/RefCounted.h>
#include <wtf/HashMap.h>
#include <wtf/text/CString.h>
#include <wtf/text/StringHash.h>

namespace WebCore {

class DataObjectClutter : public RefCounted<DataObjectClutter> {
public:
    static PassRefPtr<DataObjectClutter> create()
    {
        return adoptRef(new DataObjectClutter());
    }

    ListHashSet<String> types() const;

    const KURL url() const { return m_url; }
    const String uriList() const { return m_data.get("text/uri-list"); }
    const Vector<String> filenames() const { return m_filenames; }
    void setRange(PassRefPtr<Range> newRange) { m_range = newRange; }
    void setURL(const KURL&, const String&);
    bool hasData() const { return m_range || !m_data.isEmpty(); }
    bool hasText() const { return m_range || m_data.contains("text/plain"); }
    bool hasMarkup() const { return m_range || m_data.contains("text/html"); }
    bool hasURIList() const { return m_data.contains("text/uri-list"); }
    bool hasURL() const { return !m_url.isEmpty() && m_url.isValid(); }
    bool hasFilenames() const { return !m_filenames.isEmpty(); }
    void clearURIList() { m_data.remove("text/uri-list"); }
    void clearURL() { m_url = KURL(); }

    String data(const String&) const;
    String text() const;
    String markup() const;
    void setData(const String&, const String&);
    void setText(const String&);
    void setMarkup(const String&);
    void setURIList(const String&);
    String urlLabel() const;

    void clearAllExceptFilenames();
    void clearAll();
    void clearText();
    void clearMarkup();

private:
    HashMap<String,String> m_data;
    KURL m_url;
    Vector<String> m_filenames;
    RefPtr<Range> m_range;
};

}

#endif // DataObjectClutter_h
