/*
* Copyright (C) 2006 Michael Emmel mike.emmel@gmail.com
* Copyright (C) 2010 Collabora Ltd.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions
* are met:
* 1. Redistributions of source code must retain the above copyright
*    notice, this list of conditions and the following disclaimer.
* 2. Redistributions in binary form must reproduce the above copyright
*    notice, this list of conditions and the following disclaimer in the
*    documentation and/or other materials provided with the distribution.
*
* THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
* EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
* PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
* OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
* (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
* OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "config.h"
#include "PlatformMouseEvent.h"

#include <wtf/Assertions.h>
#include "UtilClutter.h"
#include <clutter/clutter.h>
#include <clutter/x11/clutter-x11.h>
#include <webkit/webkitgwebframeprivate.h>
#include <webkit/webkitiwebview.h>

#undef Font
#undef Auto
#undef Simple
#undef Complex

namespace WebCore {

// Keep this in sync with the other platform event constructors
PlatformMouseEvent::PlatformMouseEvent(ClutterActor* actor, const ClutterButtonEvent* event)
{
    m_timestamp = event->time;

    UtilClutter::convertEventCoords(actor, event->x, event->y, m_position, m_globalPosition);

    m_modifiers = 0;
    if (event->modifier_state & CLUTTER_SHIFT_MASK)
        m_modifiers |= ShiftKey;
    if (event->modifier_state & CLUTTER_CONTROL_MASK)
        m_modifiers |= CtrlKey;
    if (event->modifier_state & CLUTTER_MOD1_MASK)
        m_modifiers |= AltKey;
    if (event->modifier_state & CLUTTER_META_MASK)
        m_modifiers |= MetaKey;

    switch (event->type) {
    case CLUTTER_BUTTON_PRESS:
    case CLUTTER_BUTTON_RELEASE:
        m_type = PlatformEvent::MousePressed;
        m_clickCount = event->click_count;
        if (event->type == CLUTTER_BUTTON_RELEASE) {
            m_type = PlatformEvent::MouseReleased;
            m_clickCount = 0;
        }

        if (event->button == 1)
            m_button = LeftButton;
        else if (event->button == 2)
            m_button = MiddleButton;
        else if (event->button == 3)
            m_button = RightButton;
        break;
    default:
        ASSERT_NOT_REACHED();
    };
}

PlatformMouseEvent::PlatformMouseEvent(ClutterActor* actor, const ClutterMotionEvent* event)
{
    m_timestamp = event->time;

    UtilClutter::convertEventCoords(actor, event->x, event->y, m_position, m_globalPosition);

    m_modifiers = 0;
    if (event->modifier_state & CLUTTER_SHIFT_MASK)
        m_modifiers |= PlatformEvent::ShiftKey;
    if (event->modifier_state & CLUTTER_CONTROL_MASK)
        m_modifiers |= PlatformEvent::CtrlKey;
    if (event->modifier_state & CLUTTER_MOD1_MASK)
        m_modifiers |= PlatformEvent::AltKey;
    if (event->modifier_state & CLUTTER_META_MASK)
        m_modifiers |= PlatformEvent::MetaKey;

    m_type = PlatformEvent::MouseMoved;
    m_button = NoButton;
    m_clickCount = 0;

    if (event->modifier_state & CLUTTER_BUTTON1_MASK)
        m_button = LeftButton;
    else if (event->modifier_state & CLUTTER_BUTTON2_MASK)
        m_button = MiddleButton;
    else if (event->modifier_state & CLUTTER_BUTTON3_MASK)
        m_button = RightButton;
}

}
