/*
 * This file is part of the popup menu implementation for <select> elements in WebCore.
 *
 * Copyright (C) 2010 Collabora Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 *
 */

#include "config.h"
#include "PopupMenuClutter.h"

#include "FrameView.h"
#include <wtf/text/CString.h>

#include "webkit/webkitipopupmenu.h"

namespace WebCore {

PopupMenuClutter::PopupMenuClutter(PopupMenuClient* client, WebKitIPopupMenu* popup)
    : m_popupClient(client),
      m_popup(popup),
      m_initialized(false)
{
}

PopupMenuClutter::~PopupMenuClutter()
{
    if (m_popup) {
        g_signal_handlers_disconnect_matched(m_popup.get(), G_SIGNAL_MATCH_DATA, 0, 0, 0, 0, this);
        hide();
    }
}

void PopupMenuClutter::show(const IntRect& rect, FrameView* view, int index)
{
    ASSERT(client());
    ASSERT(m_popup);

    g_signal_connect(m_popup.get(), "hidden", G_CALLBACK(menuHidden), this);
    g_signal_connect(m_popup.get(), "item-activated", G_CALLBACK(menuItemActivated), this);

    if (!m_initialized) {
        const int size = client()->listSize();
        for (int i = 0; i < size; i++) {
            WebKitIPopupMenuItemData data;
            if (client()->itemIsSeparator(i)) {
                data.type = popupMenuSeparator;
                data.text = 0;
            } else {
                data.type = popupMenuItem;
                data.text = g_strdup(client()->itemText(i).utf8().data());
            }

            webkit_ipopup_menu_add_item(m_popup.get(), i, &data);

            if (data.text)
                g_free(data.text);
        }
        m_initialized = true;
    }
    webkit_ipopup_menu_show(m_popup.get(), rect.x(), rect.y());
}

void PopupMenuClutter::hide()
{
    ASSERT(m_popup);
    webkit_ipopup_menu_hide(m_popup.get());
}

void PopupMenuClutter::updateFromElement()
{
    client()->setTextFromItem(client()->selectedIndex());
}

void PopupMenuClutter::disconnectClient()
{
    m_popupClient = 0;
}

void PopupMenuClutter::menuItemActivated(WebKitIPopupMenu* popup, gint index, PopupMenuClutter* that)
{
    ASSERT(that->client());
    that->client()->valueChanged(index);
}

void PopupMenuClutter::menuHidden(WebKitIPopupMenu*, PopupMenuClutter* that)
{
    ASSERT(that->client());
    that->client()->popupDidHide();
}

}

