/*
 * Copyright (C) 2010 Collabora Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public License
 *  as published by the Free Software Foundation; either version 2.1 of
 *  the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301 USA
 */

#ifndef ScrollbarThemeClutter_h
#define ScrollbarThemeClutter_h

#include "ScrollbarThemeComposite.h"

namespace WebCore {

class ScrollbarThemeClutter : public ScrollbarThemeComposite {
public:
    ScrollbarThemeClutter();
    virtual int scrollbarThickness(ScrollbarControlSize = RegularScrollbar);

    virtual bool supportsControlTints() const { return false; }

    virtual double initialAutoscrollTimerDelay();
    virtual double autoscrollTimerDelay();

    virtual void registerScrollbar(ScrollbarThemeClient*);
    virtual void unregisterScrollbar(ScrollbarThemeClient*);

protected:
    virtual bool hasButtons(ScrollbarThemeClient*);
    virtual bool hasThumb(ScrollbarThemeClient*);

    virtual IntSize buttonSize(ScrollbarThemeClient*);
    virtual IntRect backButtonRect(ScrollbarThemeClient*, ScrollbarPart, bool painting = false);
    virtual IntRect forwardButtonRect(ScrollbarThemeClient*, ScrollbarPart, bool painting = false);
    virtual IntRect trackRect(ScrollbarThemeClient*, bool painting = false);

    virtual int maxOverlapBetweenPages() { return 40; }

    virtual int minimumThumbLength(ScrollbarThemeClient*);

    virtual void paintScrollbarBackground(GraphicsContext*, ScrollbarThemeClient*);
    virtual void paintTrackBackground(GraphicsContext*, ScrollbarThemeClient*, const IntRect&);
    virtual void paintTrackPiece(GraphicsContext*, ScrollbarThemeClient*, const IntRect&, ScrollbarPart);
    virtual void paintButton(GraphicsContext*, ScrollbarThemeClient*, const IntRect&, ScrollbarPart);
    virtual void paintThumb(GraphicsContext*, ScrollbarThemeClient*, const IntRect&);

    virtual bool shouldCenterOnThumb(ScrollbarThemeClient*, const PlatformMouseEvent&);

    void updateThemeProperties();

private:
    int m_thumbFatness;
    int m_troughBorderWidth;
    int m_stepperSize;
    int m_stepperSpacing;
    int m_minThumbLength;
    bool m_troughUnderSteppers;
    bool m_hasForwardButtonStartPart;
    bool m_hasBackButtonEndPart;
    bool m_hasButtons;
};

}

#endif
