/*
 * Copyright (C) 2010 Collabora Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include "config.h"
#include "UtilClutter.h"

#include "FloatPoint.h"
#include <webkit/webkitwebpageprivate.h>
#include <webkit/webkitiwebview.h>

namespace WebCore {

// derived from clutter/x11/clutter-stage-x11.c:_clutter_stage_x11_get_root_coords()
bool UtilClutter::stageX11GetRootCoords(ClutterStage *stage, IntPoint position, IntPoint &root)
{
    int return_val;
    Window child;
    int tx, ty;

    return_val = XTranslateCoordinates (clutter_x11_get_default_display(),
                                        clutter_x11_get_stage_window(stage),
                                        clutter_x11_get_root_window(),
                                        position.x(), position.y(), &tx, &ty,
                                        &child);
    root = IntPoint(tx, ty);

    return (return_val == 0);
}

void UtilClutter::applyInverseChildTransform(ClutterActor* actor, float x, float y, IntPoint& actorPosition)
{
    gfloat actorZ=0, actorW=1;

    // Account for scroll and scale by inverting the child transform
    // applied in mx-web-view.cpp:update_child_transform()
    ClutterMatrix childTransform, inverseChildTransform;
    clutter_actor_get_child_transform(actor, &childTransform);
    cogl_matrix_get_inverse(&childTransform, &inverseChildTransform);
    cogl_matrix_transform_point(&inverseChildTransform, &x, &y, &actorZ, &actorW);

    WebKitWebPage* webPage = webkit_iweb_view_get_page(WEBKIT_IWEB_VIEW(actor));
    WebKitWebFrame* mainFrame = webkit_web_page_get_main_frame(webPage);
    gfloat zoomFactor = webkit_web_frame_get_contents_scale(mainFrame);
    x /= zoomFactor;
    y /= zoomFactor;

    // Not that we expect perspective transformations, but still apply
    // the homogeneous divide
    x /= actorW;
    y /= actorW;

    actorPosition = roundedIntPoint(FloatPoint(x, y));
}

void UtilClutter::convertEventCoords(ClutterActor* actor, float x, float y, IntPoint& actorPosition, IntPoint &globalPosition)
{
    gfloat actorX, actorY;
    clutter_actor_transform_stage_point(actor, x, y, &actorX, &actorY);

    UtilClutter::applyInverseChildTransform(actor, actorX, actorY, actorPosition);

    ClutterStage *stage = CLUTTER_STAGE(clutter_actor_get_stage(actor));
    UtilClutter::stageX11GetRootCoords(stage, roundedIntPoint(FloatPoint(x, y)), globalPosition);
}

}
