/*
 * Copyright (C) 2011 Igalia S.L.
 * Copyright (C) 2012 Collabora Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 *  Boston, MA 02110-1301 USA
 */

#ifndef GraphicsContext3DPrivate_h
#define GraphicsContext3DPrivate_h

#include "GraphicsContext3D.h"
#include <wtf/PassOwnPtr.h>
#include "GRefPtrClutter.h"

#include <cogl/cogl.h>
#include <cogl/cogl-gles2.h>

namespace WebCore {

class GraphicsContext3DPrivate {
public:
    static PassOwnPtr<GraphicsContext3DPrivate> create(GraphicsContext3D* context);
    ~GraphicsContext3DPrivate();

    static void popGraphicsContext();
    static const CoglGLES2Vtable* openGLFunctionTable();
    static bool isCOGLES2Supported();

    bool makeContextCurrent();
    ClutterActor* platformLayer();
    PlatformGraphicsContext3D platformContext();
    void reshape(int width, int height);
    void markLayerComposited();

private:
    GraphicsContext3DPrivate(GraphicsContext3D* context);

    static GraphicsContext3DPrivate* s_current_context;

    GraphicsContext3D* m_context;
    CoglHandle m_texture;
#if USE(OPENGL_ES_2)
    CoglGLES2Context *m_gl_context;
#else
#error "WebGL will only work with OpenGLES2, in WebKit Clutter"
#endif
    CoglFramebuffer *m_framebuffer;

    int m_currentWidth, m_currentHeight;

    bool m_needsClearing;

    GRefPtr<ClutterActor> m_transformLayer;

    bool ensureGLContext();
    void ensureFBOs();
    void deleteFBOs();

    //static void cleanupActiveContextsAtExit();
};

}

#endif // GraphicsContext3DPrivate_h
