/*
 * Copyright 2011 Collabora Limited
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef GraphicsLayerActor_h
#define GraphicsLayerActor_h

#include "config.h"

#if USE(ACCELERATED_COMPOSITING)

#include "GraphicsLayerClutter.h"

#include <clutter/clutter.h>
#include <cogl/cogl.h>
#include <glib-object.h>

G_BEGIN_DECLS

#define GRAPHICS_LAYER_TYPE_ACTOR graphics_layer_actor_get_type()

#define GRAPHICS_LAYER_ACTOR(obj) \
    (G_TYPE_CHECK_INSTANCE_CAST((obj),                 \
                                GRAPHICS_LAYER_TYPE_ACTOR, GraphicsLayerActor))

#define GRAPHICS_LAYER_ACTOR_CLASS(klass) \
    (G_TYPE_CHECK_CLASS_CAST((klass),                  \
                             GRAPHICS_LAYER_TYPE_ACTOR, GraphicsLayerActorClass))

#define GRAPHICS_LAYER_IS_ACTOR(obj) \
    (G_TYPE_CHECK_INSTANCE_TYPE((obj),         \
                                GRAPHICS_LAYER_TYPE_ACTOR))

#define GRAPHICS_LAYER_IS_ACTOR_CLASS(klass) \
    (G_TYPE_CHECK_CLASS_TYPE((klass),       \
                             GRAPHICS_LAYER_TYPE_ACTOR))

#define GRAPHICS_LAYER_ACTOR_GET_CLASS(obj) \
    (G_TYPE_INSTANCE_GET_CLASS((obj),                  \
                               GRAPHICS_LAYER_TYPE_ACTOR, GraphicsLayerActorClass))

typedef struct _GraphicsLayerActor GraphicsLayerActor;
typedef struct _GraphicsLayerActorClass GraphicsLayerActorClass;
typedef struct _GraphicsLayerActorPrivate GraphicsLayerActorPrivate;

/**
 * GraphicsLayerActor:
 *
 * The contents of this structure are private and should only be accessed
 * through the public API.
 */
struct _GraphicsLayerActor {
    ClutterRectangle parent;

    GraphicsLayerActorPrivate *priv;
};

struct _GraphicsLayerActorClass {
    ClutterRectangleClass parent_class;

    /* padding for future expansion */
    void(*_padding_0) (void);
    void(*_padding_1) (void);
    void(*_padding_2) (void);
    void(*_padding_3) (void);
    void(*_padding_4) (void);
};

GType graphics_layer_actor_get_type(void) G_GNUC_CONST;

GraphicsLayerActor* graphicsLayerActorNew(WebCore::GraphicsLayerClutter::LayerType);
GraphicsLayerActor* graphicsLayerActorNewWithClient(WebCore::GraphicsLayerClutter::LayerType, WebCore::PlatformClutterLayerClient*);
void graphicsLayerActorSetClient(GraphicsLayerActor*, WebCore::PlatformClutterLayerClient*);
WebCore::PlatformClutterLayerClient* graphicsLayerActorGetClient(GraphicsLayerActor*);
void graphicsLayerActorRemoveAll(GraphicsLayerActor*);
cairo_surface_t* graphicsLayerActorGetSurface(GraphicsLayerActor*);
void graphicsLayerActorSetSurface(GraphicsLayerActor*, cairo_surface_t*);
void graphicsLayerActorInvalidateRectangle(GraphicsLayerActor*, const WebCore::FloatRect&);
void graphicsLayerActorSetChildrenTransform(GraphicsLayerActor*, const CoglMatrix*);
void graphicsLayerActorSetAnchorPoint(GraphicsLayerActor*, gfloat, gfloat, gfloat);
void graphicsLayerActorGetAnchorPoint(GraphicsLayerActor*, gfloat*, gfloat*, gfloat*);
void graphicsLayerActorSetScrollPosition(GraphicsLayerActor*, gfloat, gfloat); 
void graphicsLayerActorSetTranslateX(GraphicsLayerActor*, float);
float graphicsLayerActorGetTranslateX(GraphicsLayerActor*);
void graphicsLayerActorSetTranslateY(GraphicsLayerActor*, float);
float graphicsLayerActorGetTranslateY(GraphicsLayerActor*);
gint graphicsLayerActorGetCountChildren(GraphicsLayerActor*);
void graphicsLayerActorRemoveFromSuperLayer(GraphicsLayerActor*);
void graphicsLayerActorReplaceSublayer(GraphicsLayerActor*, ClutterActor*, ClutterActor*);
void graphicsLayerActorInsertSublayer(GraphicsLayerActor*, ClutterActor*, gint);
void graphicsLayerActorSetSublayers(GraphicsLayerActor*, WebCore::GraphicsLayerActorList&);
void graphicsLayerActorSetDoubleSided(GraphicsLayerActor*, gboolean);
gboolean graphicsLayerActorGetDoubleSided(GraphicsLayerActor*);
WebCore::GraphicsLayerClutter::LayerType graphicsLayerActorGetLayerType(GraphicsLayerActor*);
void graphicsLayerActorSetLayerType(GraphicsLayerActor*, WebCore::GraphicsLayerClutter::LayerType);
ClutterTexture* graphicsLayerActorGetTexture(GraphicsLayerActor*);
void graphicsLayerActorSetCoglTexture(GraphicsLayerActor* layer, CoglTexture *texture);
bool graphicsLayerActorGetDrawsContent(GraphicsLayerActor*);
void graphicsLayerActorSetDrawsContent(GraphicsLayerActor*, bool drawsContent);
void graphicsLayerActorSetFlatten(GraphicsLayerActor*, bool flatten);
void graphicsLayerActorSetMasksToBounds(GraphicsLayerActor*, bool maskToBounds);
gfloat graphicsLayerActorGetDepth(GraphicsLayerActor*);

/* For Debugging */
void graphicsLayerActorSetDebugBorder(GraphicsLayerActor* actor, gfloat width, ClutterColor* color);
void graphicsLayerActorPrintChildrenLayerInformation(GraphicsLayerActor* , gint indent);

WebCore::PlatformClutterAnimation* graphicsLayerActorGetAnimationForKey(GraphicsLayerActor*, const String);

G_END_DECLS

#endif // USE(ACCELERATED_COMPOSITING)

#endif /* GraphicsLayerActor_h */
