/*
 * Copyright 2011 Collabora Limited
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#ifndef _GRAPHICS_LAYER_STAGE_H
#define _GRAPHICS_LAYER_STAGE_H

#if USE(ACCELERATED_COMPOSITING)

#include <glib-object.h>
#include <clutter/clutter.h>

G_BEGIN_DECLS

#define GRAPHICS_LAYER_TYPE_STAGE graphics_layer_stage_get_type()

#define GRAPHICS_LAYER_STAGE(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
  GRAPHICS_LAYER_TYPE_STAGE, GraphicsLayerStage))

#define GRAPHICS_LAYER_STAGE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_CAST ((klass), \
  GRAPHICS_LAYER_TYPE_STAGE, GraphicsLayerStageClass))

#define GRAPHICS_LAYER_IS_STAGE(obj) \
  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), \
  GRAPHICS_LAYER_TYPE_STAGE))

#define GRAPHICS_LAYER_IS_STAGE_CLASS(klass) \
  (G_TYPE_CHECK_CLASS_TYPE ((klass), \
  GRAPHICS_LAYER_TYPE_STAGE))

#define GRAPHICS_LAYER_STAGE_GET_CLASS(obj) \
  (G_TYPE_INSTANCE_GET_CLASS ((obj), \
  GRAPHICS_LAYER_TYPE_STAGE, GraphicsLayerStageClass))

typedef struct _GraphicsLayerStage GraphicsLayerStage;
typedef struct _GraphicsLayerStageClass GraphicsLayerStageClass;
typedef struct _GraphicsLayerStagePrivate GraphicsLayerStagePrivate;

/**
 * GraphicsLayerStage:
 *
 * The contents of this structure are private and should only be accessed
 * through the public API.
 */
struct _GraphicsLayerStage
{
  ClutterStage parent;

  GraphicsLayerStagePrivate *priv;
};

struct _GraphicsLayerStageClass
{
  ClutterStageClass parent_class;

  /*< private >*/
  /* padding for future expansion */
  gpointer _padding_dummy[31];
};

typedef enum {
     DefaultPerspectiveProjection = 0,
     CustomPerspectiveProjection,
     OrthographicProjection
} StageProjectionMode;

GType graphics_layer_stage_get_type (void) G_GNUC_CONST;

GraphicsLayerStage* graphics_layer_stage_new();
void graphics_layer_stage_set_perspective_matrix(GraphicsLayerStage*, const CoglMatrix*); 
void graphics_layer_stage_set_projection_mode(GraphicsLayerStage*, StageProjectionMode); 

G_END_DECLS

#endif // USE(ACCELERATED_COMPOSITING)

#endif /* _GRAPHICS_LAYER_STAGE_H */
