/*
 * Copyright (C) 2011 Collabora Ltd.
 * Copyright (C) 2011 Samsung Electronics
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef TileClutter_h
#define TileClutter_h

#if USE(TILED_BACKING_STORE)

#include "GRefPtrClutter.h"
#include "IntPoint.h"
#include "IntRect.h"
#include "RefPtrCairo.h"
#include "Tile.h"
#include <cairo.h>
#include <wtf/PassRefPtr.h>
#include <wtf/RefCounted.h>

typedef struct _ClutterActor ClutterActor;
typedef struct _ClutterCanvas ClutterCanvas;

namespace WebCore {

class TiledBackingStore;

class TileClutter : public Tile {
public:
    static PassRefPtr<Tile> create(TiledBackingStore* backingStore, const Coordinate& tileCoordinate)
    {
        return adoptRef(new TileClutter(backingStore, tileCoordinate));
    }
    virtual ~TileClutter();

    virtual bool isDirty() const { return m_dirty; };
    virtual bool isEmpty() const { return m_empty; };

    virtual void invalidate(const IntRect&) { m_dirty = true; m_empty = false; };
    virtual Vector<IntRect> updateBackBuffer();
    virtual void swapBackBufferToFront() { m_empty = false; };
    virtual bool isReadyToPaint() const { return m_actor; };
    virtual void draw(ClutterCanvas*, cairo_t*, int width, int height);
    virtual void paint(GraphicsContext*, const IntRect&) { };

    virtual const Tile::Coordinate& coordinate() const { return m_coordinate; }
    virtual const IntRect& rect() const { return m_rect; }
    virtual void resize(const WebCore::IntSize&);

    virtual ClutterActor* actor() const { return m_actor.get(); };

protected:
    TileClutter(TiledBackingStore*, const Coordinate&);

    TiledBackingStore* m_backingStore;
    Coordinate m_coordinate;
    IntRect m_rect;

    GRefPtr<ClutterActor> m_actor;
    bool m_dirty;
    bool m_empty;
};

}
#endif
#endif

