/*
 * Copyright (C) 2007 Apple Inc.  All rights reserved.
 * Copyright (C) 2007 Holger Hans Peter Freyther
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Apple, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef DragClientClutter_h
#define DragClientClutter_h

#include "DragClientGObject.h"
#include "DataObjectClutter.h"
#include "DragActions.h"
#include <wtf/gobject/GRefPtr.h>
#include "RefPtrCairo.h"
#include <clutter/clutter.h>

typedef struct _WebKitWebPage WebKitWebPage;

namespace WebKit {

    class DragClientClutter : public DragClientGObject {
    public:
        DragClientClutter(WebKitWebPage*);
        virtual void startDrag(WebCore::DragImageRef dragImage, const WebCore::IntPoint& dragImageOrigin, const WebCore::IntPoint& eventPos, WebCore::Clipboard*, WebCore::Frame*, bool linkDrag = false) OVERRIDE;

    private:
        static gboolean onCapturedEvent(ClutterActor *stage, const ClutterEvent *event, gpointer data);
        static gboolean onCanvasDraw(ClutterCanvas *canvas, cairo_t *cr, int width, int height, gpointer data);
        void onDragUpdate(const ClutterEvent *event);
        void onDragEnd(const ClutterEvent *event, bool success);

    private:
        WebKitWebPage* m_webPage;
        ClutterInputDevice *m_startDragEventDevice;
        const ClutterEventSequence *m_startDragEventSequence;
        gulong m_onCapturedEventId;
        RefPtr<WebCore::DataObjectClutter> m_dataObject;
        GRefPtr<ClutterActor> m_handleActor;
        RefPtr<cairo_surface_t> m_handleImage;
        WebCore::DragOperation m_dragOperation;
        WebCore::IntSize m_dragImageOffset;
    };
}

#endif
