/*
 *  Copyright (C) 2011 Collabora Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef DumpRenderTreeSupportClutter_h
#define DumpRenderTreeSupportClutter_h

#include "webkit/webkitgwebframe.h"
#include <glib.h>
#include <wtf/HashMap.h>
#include <wtf/text/CString.h>

typedef struct _DRTSupportGObject DRTSupportGObject;

namespace WebCore {
class DOMWrapperWorld;
}

class DumpRenderTreeSupportClutter {

public:
    DumpRenderTreeSupportClutter();
    ~DumpRenderTreeSupportClutter();

    static void setDumpRenderTreeModeEnabled(bool);
    static bool dumpRenderTreeModeEnabled();
    static DRTSupportGObject* gobject();

    static GSList* getFrameChildren(WebKitWebFrame*);
    static WTF::CString getInnerText(WebKitWebFrame*);
    static WTF::CString dumpRenderTree(WebKitWebFrame*, bool);
    static WTF::CString counterValueForElementById(WebKitWebFrame*, const char* id);
    static unsigned int getPendingUnloadEventCount(WebKitWebFrame*);
    static bool frameIsProcessingUserGesture(WebKitWebFrame*);

    static unsigned int numberOfActiveAnimations(WebKitWebFrame*);
    static void suspendAnimations(WebKitWebFrame*);
    static void resumeAnimations(WebKitWebFrame*);
    static bool pauseAnimation(WebKitWebFrame*, const char* name, double time, const char* element);
    static bool pauseTransition(WebKitWebFrame*, const char* name, double time, const char* element);

    static bool elementDoesAutoCompleteForElementWithId(WebKitWebFrame*, const char* name);

    static WTF::CString plainText(JSContextRef, JSValueRef);
    static bool findString(WebKitWebFrame*, const char* target, GList* optionsList);

    static void addUserScript(WebKitWebFrame*, const char* source, bool runAtStart, bool allFrames);
    static void addUserStyleSheet(WebKitWebFrame*, const char* source, bool allFrames);
    static void removeAllUserContent(WebKitWebFrame*);

    static WTF::CString markerTextForListItem(WebKitWebFrame*, JSContextRef, JSValueRef nodeObject);

    static void setLinksIncludedInFocusChain(bool);
    static bool linksIncludedInFocusChain();
    static JSValueRef nodesFromRect(JSContextRef context, JSValueRef value, int x, int y, unsigned top, unsigned right, unsigned bottom, unsigned left, bool ignoreClipping);
    static JSValueRef computedStyleIncludingVisitedInfo(JSContextRef, JSValueRef);

    static int pageNumberForElementById(WebKitWebFrame*, const char* id, float pageWidth, float pageHeight);
    static int numberOfPagesForFrame(WebKitWebFrame*, float pageWidth, float pageHeight);
    static WTF::CString pageProperty(WebKitWebFrame*, const char* propertyName, int pageNumber);
    static bool isPageBoxVisible(WebKitWebFrame*, int pageNumber);
    static WTF::CString pageSizeAndMarginsInPixels(WebKitWebFrame*, int pageNumber, int width, int height, int marginTop, int marginRight, int marginBottom, int marginLeft);

    static void clearScriptWorlds();
    static void evaluateScriptInIsolatedWorld(WebKitWebFrame*, unsigned, const char*);

    static void resetGeolocationClientMock(WebKitWebPage*);
    static void setMockGeolocationPermission(WebKitWebPage*, bool allowed);
    static void setMockGeolocationPosition(WebKitWebPage*, double latitude, double longitude, double accuracy);
    static void setMockGeolocationPositionUnavailableError(WebKitWebPage*, const gchar* errorMessage);
    static int numberOfPendingGeolocationPermissionRequests(WebKitWebPage*);

private:
    static DRTSupportGObject* s_gobject;

    static bool s_drtRun;
    static bool s_linksIncludedInTabChain;

    static WTF::HashMap<unsigned, RefPtr<WebCore::DOMWrapperWorld> > s_worldMap;
};

#endif
