/*
 *  Copyright (C) 2007, 2008 Alp Toker <alp@atoker.com>
 *  Copyright (C) 2007, 2008, 2009 Holger Hans Peter Freyther
 *  Copyright (C) 2007 Christian Dywan <christian@twotoasts.de>
 *  Copyright (C) 2008, 2009 Collabora Ltd.  All rights reserved.
 *  Copyright (C) 2009, 2010 Gustavo Noronha Silva <gns@gnome.org>
 *  Copyright (C) Research In Motion Limited 2009. All rights reserved.
 *  Copyright (C) 2010 Igalia S.L.
 *  Copyright (C) 2010 Collabora Ltd.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "config.h"
#include "FrameLoaderClientGObject.h"

// FIXME: Needs to be imported before WebCore::Status gets defined
#include "PluginDatabase.h"

#include "CachedFrame.h"
#include "Color.h"
#include "DocumentLoader.h"
#include "FormState.h"
#include "FrameLoader.h"
#include "FrameNetworkingContext.h"
#include "FrameView.h"
#include "FrameTree.h"
#include <wtf/gobject/GOwnPtr.h>
#include <wtf/gobject/GRefPtr.h>
#include "HTMLAppletElement.h"
#include "HTMLFormElement.h"
#include "HTMLFrameElement.h"
#include "HTMLFrameOwnerElement.h"
#include "HTMLNames.h"
#include "HTMLPlugInElement.h"
#include "JSDOMWindow.h"
#include "Language.h"
#include "MIMETypeRegistry.h"
#include "MouseEvent.h"
#include "NotImplemented.h"
#include "Page.h"
#include "RenderPart.h"
#include "ResourceHandle.h"
#include "ResourceRequest.h"
#include "ProgressTracker.h"
#include "JSDOMBinding.h"
#include "ScriptController.h"
#include "webkitclutterinclude.h"
#include "webkitgwebframeprivate.h"
#include "webkitiwebview.h"
#include "webkitwebpageprivate.h"
#include "webkitwebsettings.h"
#include <gio/gio.h>
#include <glib.h>
#include <glib/gi18n-lib.h>
#include <stdio.h>
#include <wtf/text/CString.h>

using namespace WebCore;

namespace WebKit {

char* FrameLoaderClient::toolkitLoadErrorPage()
{
    notImplemented();
    return 0;
}

void FrameLoaderClient::toolkitPostCommitFrameViewSetup(FrameView* frameView, bool resetScrolling)
{
    webkit_iweb_view_post_commit_setup(getViewFromFrame(m_frame), resetScrolling);
}

IntSize FrameLoaderClient::toolkitGetAllocation()
{
    // FIXME: this might be called before the actor has been assigned to the page. Should we make
    // the actor a construct-only proprety?
    ClutterActor* webActor = CLUTTER_ACTOR(getViewFromFrame(m_frame));

    if (!webActor)
        return IntSize();

    ClutterActorBox actorBox;
    clutter_actor_get_allocation_box(CLUTTER_ACTOR(webActor), &actorBox);
    return IntSize(actorBox.x2 - actorBox.x1, actorBox.y2 - actorBox.y1);
}

bool FrameLoaderClient::toolkitGetTransparent()
{
    notImplemented();
    return false;
}

int FrameLoaderClient::toolkitGetModifierFlags(UIEventWithKeyState* keyStateEvent)
{
    notImplemented();
    return 0;
}

PassRefPtr<Widget> FrameLoaderClient::createPlugin(const IntSize& pluginSize, HTMLPlugInElement* element, const KURL& url, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType, bool loadManually)
{
    RefPtr<PluginView> pluginView = PluginView::create(core(m_frame), pluginSize, element, url, paramNames, paramValues, mimeType, loadManually);
    if (pluginView && pluginView->status() == PluginStatusLoadedSuccessfully)
        return pluginView;

    return 0;
}

void FrameLoaderClient::didTransferChildFrameToNewDocument(WebCore::Page*)
{
    notImplemented();
}

void FrameLoaderClient::toolkitDispatchDidClearWindowObjectInWorld(JSGlobalContextRef, JSObjectRef)
{
    notImplemented();
}

void FrameLoaderClient::toolkitDispatchDidCommitLoad()
{
    notImplemented();
}

Frame* FrameLoaderClient::dispatchCreatePage(const WebCore::NavigationAction&)
{
    WebKitWebPage* page = webkit_web_frame_get_page(m_frame);
    WebKitWebPage* newPage;

    g_signal_emit_by_name(page, "create-window", m_frame, &newPage);
    if (!newPage)
        return 0;

    return core(newPage->priv->mainFrame);
}

void FrameLoaderClient::savePlatformDataToCachedFrame(CachedFrame* cachedFrame)
{
    notImplemented();
}

PassRefPtr<FrameNetworkingContext> FrameLoaderClient::createNetworkingContext()
{
    notImplemented();
    return 0;
}

}
