/*
 * Copyright (C) 2012 Collabora Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include <clutter/clutter.h>
#include <libsoup/soup.h>
#include <string.h>
#include <webkit/webkitwebpage.h>
#include <webkit/webkitiwebview.h>
#include <webkit/vanilla/webkitwebactor.h>

extern void webkit_init();

#if GLIB_CHECK_VERSION(2, 16, 0)

const char* html_string = \
"<body></body>\n" \
"<script>\n" \
"  var i=0;\n" \
"  while(i < 10000000000000000000000) {\n" \
"    i=i+1;\n" \
"    if (i%9) {\n" \
"      var p = document.createElement('p');\n" \
"      p.innerText = document.body.clientHeight;\n" \
"      document.body.appendChild(p);\n" \
"    }\n" \
"  }\n" \
"</script>\n";

static gboolean shouldInterruptScript(WebKitWebPage* webPage, GMainLoop* loop)
{
    g_main_loop_quit(loop);
    return TRUE;
}

static void test_jsinterrupt(void)
{
    ClutterActor* webActor = webkit_web_actor_new();

    WebKitWebPage* webPage = webkit_iweb_view_get_page(WEBKIT_IWEB_VIEW(webActor));

    GMainLoop* loop = g_main_loop_new(NULL, TRUE);

    g_signal_connect(webPage, "should-interrupt-script", G_CALLBACK(shouldInterruptScript), loop);

    webkit_iweb_view_load_string(WEBKIT_IWEB_VIEW(webActor), html_string, "text/html", NULL, "file://");

    g_main_loop_run(loop);
}

int main(int argc, char** argv)
{
    g_test_init(&argc, &argv, NULL);
    if (clutter_init(&argc, &argv) != CLUTTER_INIT_SUCCESS) {
        g_error ("Failed to initialized clutter.");
        return 1;
    }
    webkit_init();

    g_test_bug_base("https://bugs.webkit.org/");
    g_test_add_func("/webkit/jsinterrupt", test_jsinterrupt);
    return g_test_run();
}

#else
int main(int argc, char** argv)
{
    g_critical("You will need at least glib-2.16.0 to run the unit tests. Doing nothing now.");
    return 0;
}

#endif
