/*
 * Copyright 2010 Collabora Limited
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU Lesser General Public License,
 * version 2.1, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for
 * more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 */

#include "webkit-mx-popup-menu.h"
#include <webkit/webkitipopupmenu.h>

#include <clutter/clutter.h>
#include <mx/mx.h>

static void webkit_ipopup_menu_iface_init(WebKitIPopupMenuInterface* iface);

G_DEFINE_TYPE_WITH_CODE(WebKitMxPopupMenu, webkit_mx_popup_menu, MX_TYPE_DIALOG,
                        G_IMPLEMENT_INTERFACE(WEBKIT_TYPE_IPOPUP_MENU,
                                              webkit_ipopup_menu_iface_init))

#define WEBKIT_MX_POPUP_MENU_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), WEBKIT_MX_TYPE_POPUP_MENU, WebKitMxPopupMenuPrivate))

struct _WebKitMxPopupMenuPrivate
{
    guint count;
    ClutterActor* scrollview;
    MxBoxLayout* layout;
    gboolean initialized;
};

static void
webkit_mx_popup_menu_class_init (WebKitMxPopupMenuClass *klass)
{
    g_type_class_add_private (klass, sizeof (WebKitMxPopupMenuPrivate));
}

static void
webkit_mx_popup_menu_init (WebKitMxPopupMenu *self)
{
    WebKitMxPopupMenuPrivate* priv = self->priv = WEBKIT_MX_POPUP_MENU_GET_PRIVATE(self);

    priv->layout = MX_BOX_LAYOUT(mx_box_layout_new());
    mx_box_layout_set_orientation(priv->layout, MX_ORIENTATION_VERTICAL);
}

WebKitMxPopupMenu*
webkit_mx_popup_menu_new(void)
{
    return WEBKIT_MX_POPUP_MENU(g_object_new(WEBKIT_MX_TYPE_POPUP_MENU, NULL));
}

static void webkit_mx_popup_menu_show(WebKitIPopupMenu* popup, guint x, guint y)
{
    WebKitMxPopupMenu* self = WEBKIT_MX_POPUP_MENU(popup);
    WebKitMxPopupMenuPrivate* priv = WEBKIT_MX_POPUP_MENU_GET_PRIVATE(self);

    if (!priv->initialized) {
        priv->scrollview = mx_scroll_view_new();
        clutter_actor_add_child(CLUTTER_ACTOR(self), priv->scrollview);
        clutter_actor_add_child(priv->scrollview, CLUTTER_ACTOR(priv->layout));
        priv->initialized = TRUE;
    }

    ClutterActor* child = priv->scrollview;
    ClutterActor* stage = clutter_actor_get_stage(child);

    float stage_height = clutter_actor_get_height(stage);
    float stage_width = clutter_actor_get_width(stage);

    clutter_actor_set_size(child, stage_width * 0.8, stage_height * 0.7);

    clutter_actor_show(popup);
    return;
}

static void webkit_mx_popup_menu_hide(WebKitIPopupMenu* popup)
{
    clutter_actor_hide(CLUTTER_ACTOR(popup));
    g_signal_emit_by_name(popup, "hidden", NULL);
    return;
}

static void webkit_mx_popup_menu_item_clicked(MxButton* button,
                                              WebKitIPopupMenu* popup)
{
    gint index = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(button), "index"));
    g_signal_emit_by_name(popup, "item-activated", index, NULL);
    webkit_mx_popup_menu_hide(popup);
}

static void webkit_mx_popup_menu_add_item(WebKitIPopupMenu* popup,
                                          gint index,
                                          WebKitIPopupMenuItemData* item)
{
    WebKitMxPopupMenuPrivate* priv = WEBKIT_MX_POPUP_MENU_GET_PRIVATE(WEBKIT_MX_POPUP_MENU(popup));
    if (item->type != popupMenuItem)
        return;

    ClutterActor* button = mx_button_new_with_label(item->text);
    g_object_set_data(G_OBJECT(button), "index", GINT_TO_POINTER(index));
    g_signal_connect(button, "clicked", (GCallback)webkit_mx_popup_menu_item_clicked, popup);
    mx_box_layout_insert_actor(priv->layout, button, index);
    return;
}

static void webkit_ipopup_menu_iface_init (WebKitIPopupMenuInterface* iface)
{
    iface->show = webkit_mx_popup_menu_show;
    iface->add_item = webkit_mx_popup_menu_add_item;
    iface->hide = webkit_mx_popup_menu_hide;
}
