/*
 * Copyright (C) 2010 Collabora Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef webkitipopupmenu_h
#define webkitipopupmenu_h

#include <webkit/webkitdefines.h>
#include <glib-object.h>

G_BEGIN_DECLS

#define WEBKIT_TYPE_IPOPUP_MENU                (webkit_ipopup_menu_get_type())
#define WEBKIT_IPOPUP_MENU(obj)                (G_TYPE_CHECK_INSTANCE_CAST((obj), WEBKIT_TYPE_IPOPUP_MENU, WebKitIPopupMenu))
#define WEBKIT_IS_IPOPUP_MENU(obj)             (G_TYPE_CHECK_INSTANCE_TYPE((obj), WEBKIT_TYPE_IPOPUP_MENU))
#define WEBKIT_IPOPUP_MENU_GET_INTERFACE(inst) (G_TYPE_INSTANCE_GET_INTERFACE((inst),  WEBKIT_TYPE_IPOPUP_MENU, WebKitIPopupMenuInterface))

typedef struct _WebKitIPopupMenu WebKitIPopupMenu;
typedef struct _WebKitIPopupMenuInterface WebKitIPopupMenuInterface;

/**
 * WebKitIPopupMenuItemType:
 * @popupMenuItem : Update description
 * @popupMenuSeparator: Update description
 **/
typedef enum {
    popupMenuItem = 0,
    popupMenuSeparator,
} WebKitIPopupMenuItemType;

/**
 * WebKitIPopupMenuItemData:
 * @type: Update description
 * @text: Update description
 **/
typedef struct {
    WebKitIPopupMenuItemType type;
    gchar* text;
} WebKitIPopupMenuItemData;

struct _WebKitIPopupMenuInterface {
    GTypeInterface parent;

    // FIXME: What are the positions relative to?
    void (*show)     (WebKitIPopupMenu*, guint x, guint y);
    void (*add_item) (WebKitIPopupMenu*, gint, WebKitIPopupMenuItemData*);
    void (*hide)     (WebKitIPopupMenu*);

    /* Signals */
    void (*item_activated) (WebKitIPopupMenu*, gint);
    void (*hidden)         (WebKitIPopupMenu*);

    /* Padding for future expansion */
    void (*_webkit_reserved0) (void);
    void (*_webkit_reserved1) (void);
    void (*_webkit_reserved2) (void);
    void (*_webkit_reserved3) (void);
};

WEBKIT_API GType
webkit_ipopup_menu_get_type    (void);

WEBKIT_API void
webkit_ipopup_menu_show        (WebKitIPopupMenu*, guint x, guint y);

WEBKIT_API void
webkit_ipopup_menu_add_item    (WebKitIPopupMenu*,
                                gint,
                                WebKitIPopupMenuItemData*);

WEBKIT_API void
webkit_ipopup_menu_hide        (WebKitIPopupMenu*);

G_END_DECLS

#endif
