/*
 * Copyright (C) 2010 Collabora Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef webkitirendertheme_h
#define webkitirendertheme_h

#include "webkitenumtypes.h"
#include <cairo/cairo.h>
#include <webkit/webkitdefines.h>

G_BEGIN_DECLS

#define WEBKIT_TYPE_IRENDER_THEME                (webkit_irender_theme_get_type())
#define WEBKIT_IRENDER_THEME(obj)                (G_TYPE_CHECK_INSTANCE_CAST((obj), WEBKIT_TYPE_IRENDER_THEME, WebKitIRenderTheme))
#define WEBKIT_IS_IRENDER_THEME(obj)             (G_TYPE_CHECK_INSTANCE_TYPE((obj), WEBKIT_TYPE_IRENDER_THEME))
#define WEBKIT_IRENDER_THEME_GET_INTERFACE(inst) (G_TYPE_INSTANCE_GET_INTERFACE((inst),  WEBKIT_TYPE_IRENDER_THEME, WebKitIRenderThemeInterface))

typedef struct _WebKitIRenderTheme WebKitIRenderTheme;
typedef struct _WebKitIRenderThemeInterface WebKitIRenderThemeInterface;

#include <cairo.h>
#if CAIRO_VERSION < CAIRO_VERSION_ENCODE(1, 10, 0)
/**
  * cairo_rectangle_int_t:
  * @x: x
  * @y: y
  * @width: width
  * @height: height
  **/
typedef struct _cairo_rectangle_int {
    int x, y;
    int width, height;
} cairo_rectangle_int_t;
#endif

/**
 * WebKitIRenderThemePlatformColor:
 * @platformColorActiveSelectionBackground: Text selection colors
 * @platformColorInactiveSelectionBackground: Text selection colors
 * @platformColorActiveSelectionForeground: Text selection colors
 * @platformColorInactiveSelectionForeground: Text selection colors
 * @platformColorActiveListBoxSelectionBackground: List box selection colors
 * @platformColorActiveListBoxSelectionForeground: List box selection colors
 * @platformColorInactiveListBoxSelectionBackground: List box selection colors
 * @platformColorInactiveListBoxSelectionForeground: List box selection colors
 * @platformColorActiveTextSearchHighlightColor: Highlighting colors for TextMatches
 * @platformColorInactiveTextSearchHighlightColor: Highlighting colors for TextMatches
 * @platformColorFocusRingColor: update description
 *
 * Platform colors.
 **/
typedef enum {
    /* Text selection colors */
    platformColorActiveSelectionBackground,
    platformColorInactiveSelectionBackground,
    platformColorActiveSelectionForeground,
    platformColorInactiveSelectionForeground,

    /* List box selection colors */
    platformColorActiveListBoxSelectionBackground,
    platformColorActiveListBoxSelectionForeground,
    platformColorInactiveListBoxSelectionBackground,
    platformColorInactiveListBoxSelectionForeground,

    /* Highlighting colors for TextMatches */
    platformColorActiveTextSearchHighlightColor,
    platformColorInactiveTextSearchHighlightColor,

    platformColorFocusRingColor
} WebKitIRenderThemePlatformColor;

/* Parts of this are copied/adapted from gtk2drawing.h, LGPL */
typedef enum {
    arrowUp,
    arrowDown,
    arrowLeft,
    arrowRight,
    arrowNone
} WebKitIRenderThemeArrowType;

typedef enum {
    /* First eight bits are used to pass a margin. */
    tabMarginMask = 0xFF,
    tabBottom     = 1 << 8,
    tabFirst      = 1 << 9,
    tabSelected   = 1 << 10
} WebKitIRenderThemeTabFlags;

typedef enum {
    stepperDown     = 1 << 0,
    stepperBottom   = 1 << 1,
    stepperVertical = 1 << 2
} WebKitIRenderThemeScrollbarButtonFlags;

typedef enum {
    mediaPlaying = 1 << 0,
    mediaMuted = 1 << 2,
} WebKitIRenderThemeMediaFlags;

typedef enum {
    toplevelMenuItem = 1 << 0
} WebKitIRenderThemeMenuItemFlags;

typedef enum {
    widgetChecked = 1,
    widgetInconsistent = 1 << 1
} WebKitIRenderThemeCheckFlags;

/**
 * WebKitIRenderThemeWidgetType:
 * @widgetButton: Paints a button.
 * @widgetCheckButton: Paints a check button. flags is a boolean, 1=checked, 0=not checked.
 * @widgetRadioButton: Paints a radio button. flags is a boolean, 1=checked, 0=not checked.
 * @widgetScrollbarButton: Paints the button of a scrollbar. flags is a WebKitIRenderThemeArrowType giving the arrow direction.
 * @widgetScrollbarTrackHorizontal: Paints the trough (track) of a scrollbar.
 * @widgetScrollbarTrackVertical: Paints the trough (track) of a scrollbar
 * @widgetScrollbarThumbHorizontal: Paints the slider (thumb) of a scrollbar.
 * @widgetScrollbarThumbVertical: Paints the slider (thumb) of a scrollbar.
 * @widgetScrolledWindow: Paints the background of a scrolled window
 * @widgetScaleHorizontal: Paints a scale.
 * @widgetScaleVertical: Paints a scale.
 * @widgetScaleThumbHorizontal: Paints a scale thumb.
 * @widgetScaleThumbVertical: Paints a scale thumb.
 * @widgetSpinButton: Paints a spin button
 * @widgetSpinButtonUp: Paints a spin button
 * @widgetSpinButtonDown: Paints a spin button
 * @widgetSpinButtonEntry: Paints a spin button
 * @widgetSearchFieldDecoration: Paints the decoration on search fields
 * @widgetSearchFieldResultsButton: Paints the results button in search fields
 * @widgetSearchFieldCancelButton: Paints the button to clear the search field
 * @widgetGripper: Paints the gripper of a handle box.
 * @widgetEntry: Paints an entry.
 * @widgetCaret: Paints the native caret (or in GTK-speak: insertion cursor)
 * @widgetDropDown: Paints a drop-down menu.
 * @widgetDropDownArrow: Paints a dropdown arrow (a button containing a down arrow).
 * @widgetDropDownEntry: Paints an entry in an editable option menu
 * @widgetCheckButtonContainer: Paints the container part of a check button
 * @widgetRadioButtonContainer: Paints the container part of a radio button.
 * @widgetCheckButtonLabel: Paints the label of a check button (focus outline)
 * @widgetRadioButtonLabel: Paints the label of a radio button (focus outline)
 * @widgetToolbar: Paints the background of a handle box.
 * @widgetToolbarSeparator: Paints a toolbar separator
 * @widgetTooltip: Paints a tooltip
 * @widgetFrame: Paints a frame (e.g. a status bar panel).
 * @widgetResizer: Paints a resize grip for a window
 * @widgetProgressBar: Paints a progress bar.
 * @widgetProgressChunk: Paints a progress chunk of a progress bar.
 * @widgetTab: Paints a tab of a notebook. flags is a WebKitIRenderThemeTabFlags, defined above.
 * @widgetTabPanels: Paints the background and border of a notebook.
 * @widgetTabScrollArrow: Paints an arrow for a notebook. flags is a WebKitIRenderThemeArrowType.
 * @widgetTreeView: Paints the background and border of a tree view
 * @widgetTreeHeaderCell: Paints treeheader cells
 * @widgetTreeHeaderSortArrow: Paints sort arrows in treeheader cells
 * @widgetTreeViewExpander: Paints an expander for a tree view
 * @widgetExpander: Paints an expander
 * @widgetMenuBar: Paints the background of the menu bar.
 * @widgetMenuPopup: Paints the background of menus, context menus.
 * @widgetMenuArroe: Paints the arrow of menuitems that contain submenus
 * @widgetToolbarButtonArrow: Paints an arrow that points down
 * @widgetMenuItem: Paints items of menubar and popups.
 * @widgetCheckMenuItem: Paints items of menubar and popups.
 * @widgetRadioMenuItem: Paints items of menubar and popups.
 * @widgetMenuSeparator: Paints items of menubar and popups.
 * @widgetSplitterHorizontal: Paints a vertical panel separator
 * @widgetSplitterVertical: Paints a horizontal panel separator
 * @widgetMediaFullscreenButton: Media-related
 * @widgetMediaMuteButton: Media-related
 * @widgetMediaPlayButton: Media-related
 * @widgetMediaSeekBackButton: Media-related
 * @widgetMediaSeekForwardButton: Media-related
 * @widgetMediaSliderTrack: Media-related
 * @widgetMediaSliderThumb: Media-related
 * @widgetWindow: Paints the background of a window, dialog or page.
 *
 * Update dscription.
 **/
typedef enum {
    /* Paints a button. */
    widgetButton,
    /* Paints a check button. flags is a boolean, 1=checked, 0=not checked. */
    widgetCheckButton,
    /* Paints a radio button. flags is a boolean, 1=checked, 0=not checked. */
    widgetRadioButton,
    /*
    Paints the button of a scrollbar. flags is a WebKitIRenderThemeArrowType
    giving the arrow direction.
    */
    widgetScrollbarButton,
    /* Paints the trough (track) of a scrollbar. */
    widgetScrollbarTrackHorizontal,
    widgetScrollbarTrackVertical,
    /* Paints the slider (thumb) of a scrollbar. */
    widgetScrollbarThumbHorizontal,
    widgetScrollbarThumbVertical,
    /* Paints the background of a scrolled window */
    widgetScrolledWindow,
    /* Paints a scale. */
    widgetScaleHorizontal,
    widgetScaleVertical,
    /* Paints a scale thumb. */
    widgetScaleThumbHorizontal,
    widgetScaleThumbVertical,
    /* Paints a spin button */
    widgetSpinButton,
    widgetSpinButtonUp,
    widgetSpinButtonDown,
    widgetSpinButtonEntry,
    /* Paints the decoration on search fields */
    widgetSearchFieldDecoration,
    /* Paints the results button in search fields */
    widgetSearchFieldResultsButton,
    /* Paints the button to clear the search field */
    widgetSearchFieldCancelButton,
    /* Paints the gripper of a handle box. */
    widgetGripper,
    /* Paints an entry. */
    widgetEntry,
    /* Paints the native caret (or in GTK-speak: insertion cursor) */
    widgetCaret,
    /* Paints a drop-down menu. */
    widgetDropDown,
    /* Paints a dropdown arrow (a button containing a down arrow). */
    widgetDropDownArrow,
    /* Paints an entry in an editable option menu */
    widgetDropDownEntry,
    /* Paints the container part of a check button. */
    widgetCheckButtonContainer,
    /* Paints the container part of a radio button. */
    widgetRadioButtonContainer,
    /* Paints the label of a check button (focus outline) */
    widgetCheckButtonLabel,
    /* Paints the label of a radio button (focus outline) */
    widgetRadioButtonLabel,
    /* Paints the background of a handle box. */
    widgetToolbar,
    /* Paints a toolbar separator */
    widgetToolbarSeparator,
    /* Paints a tooltip */
    widgetTooltip,
    /* Paints a frame (e.g. a status bar panel). */
    widgetFrame,
    /* Paints a resize grip for a window */
    widgetResizer,
    /* Paints a progress bar. */
    widgetProgressBar,
    /* Paints a progress chunk of a progress bar. */
    widgetProgressChunk,
    /* Paints a tab of a notebook. flags is a WebKitIRenderThemeTabFlags, defined above. */
    widgetTab,
    /* Paints the background and border of a notebook. */
    widgetTabPanels,
    /* Paints an arrow for a notebook. flags is a WebKitIRenderThemeArrowType. */
    widgetTabScrollArrow,
    /* Paints the background and border of a tree view */
    widgetTreeView,
    /* Paints treeheader cells */
    widgetTreeHeaderCell,
    /* Paints sort arrows in treeheader cells */
    widgetTreeHeaderSortArrow,
    /* Paints an expander for a tree view */
    widgetTreeViewExpander,
    /* Paints an expander */
    widgetExpander,
    /* Paints the background of the menu bar. */
    widgetMenuBar,
    /* Paints the background of menus, context menus. */
    widgetMenuPopup,
    /* Paints the arrow of menuitems that contain submenus */
    widgetMenuArroe,
    /* Paints an arrow that points down */
    widgetToolbarButtonArrow,
    /* Paints items of menubar and popups. */
    widgetMenuItem,
    widgetCheckMenuItem,
    widgetRadioMenuItem,
    widgetMenuSeparator,
    /* Paints a vertical panel separator */
    widgetSplitterHorizontal,
    /* Paints a horizontal panel separator */
    widgetSplitterVertical,
    /* Media-related */
    widgetMediaFullscreenButton,
    widgetMediaMuteButton,
    widgetMediaPlayButton,
    widgetMediaSeekBackButton,
    widgetMediaSeekForwardButton,
    widgetMediaSliderTrack,
    widgetMediaSliderThumb,
    /* Paints the background of a window, dialog or page. */
    widgetWindow
} WebKitIRenderThemeWidgetType;

/**
 * WebKitIRenderThemeWidgetState:
 * @active: update discription
 * @focused: update discription
 * @inHover: update discription
 * @disabled: update discription
 * @isDefault: update discription
 * @canDefault: update discription
 * @depressed: update discription
 * @curpos: curpos and maxpos are used for scrollbars
 * @maxpos: update discription
 *
 * The depressed state is for buttons which remain active for a longer period:
 * activated toggle buttons or buttons showing a popup menu.
 **/
typedef struct {
    guint8 active;
    guint8 focused;
    guint8 inHover;
    guint8 disabled;
    guint8 isDefault;
    guint8 canDefault;
    /* The depressed state is for buttons which remain active for a longer period:
     * activated toggle buttons or buttons showing a popup menu. */
    guint8 depressed;
    gint32 curpos; /* curpos and maxpos are used for scrollbars */
    gint32 maxpos;
} WebKitIRenderThemeWidgetState;

/**
 * WebKitIRenderThemeWidgetMetrics:
 * @blinkInterval: blink interval
 * @checkboxIndicatorSize: Checkbox buttons
 * @checkboxIndicatorSpacing: Checkbox buttons
 * @radioIndicatorSize: Radio buttons
 * @radioIndicatorSpacing: Radio buttons
 * @buttonInnerLeft: Button inner borders
 * @buttonInnerRight: Button inner borders
 * @buttonInnerTop: Button inner borders
 * @buttonInnerBottom: Button inner borders
 * @focusInterior: Focus metrics for a treeheadercell, button, checkbox, or radio button.
 * @focusWidth: Focus metrics for a treeheadercell, button, checkbox, or radio button.
 * @focusPad: Focus metrics for a treeheadercell, button, checkbox, or radio button.
 * @scalethumbLenght: Scale thumbs
 * @scalethumbHeigth: Scale thumbs
 * @menulistButtonWidth: Dropdown arrow button
 * @menulistButtonHeight: Dropdown arrow button
 * @scrollarrowWidth: Size of a scroll arrow widget
 * @scrollarrowHeight: Size of a scroll arrow widget
 * @downarrowWidth: Toolbar button dropdown arrow
 * @downarrowHeight: Toolbar button dropdown arrow
 * @toolbarSeparatorWidth: Toolbar separator width
 * @expanderWidth: Expander size
 * @expanderHeight: Expander size
 * @menuSeparatorHeight: Menu separator height
 * @splitterHorizontalHeight: Splitter size
 * @splitterVerticalWidth: Splitter size
 * @notebookTabThickness: Notebook tab thickness
 * @menuImages: Image in menus?
 *
 * Description.
 **/
typedef struct {
    gfloat blinkInterval;
    /* Checkbox buttons */
    gint checkboxIndicatorSize;
    gint checkboxIndicatorSpacing;

    /* Radio buttons */
    gint radioIndicatorSize;
    gint radioIndicatorSpacing;

    /* Button inner borders */
    gint buttonInnerLeft;
    gint buttonInnerRight;
    gint buttonInnerTop;
    gint buttonInnerBottom;

    /* Focus metrics for a treeheadercell, button, checkbox, or radio button. */
    gboolean focusInterior;
    gint focusWidth;
    gint focusPad;

    /* Scale thumbs */
    gint scalethumbLenght;
    gint scalethumbHeigth;

    /* Dropdown arrow button */
    gint menulistButtonWidth;
    gint menulistButtonHeight;

    /* Size of a scroll arrow widget; */
    gint scrollarrowWidth;
    gint scrollarrowHeight;

    /* Toolbar button dropdown arrow */
    gint downarrowWidth;
    gint downarrowHeight;

    /* Toolbar separator width */
    gint toolbarSeparatorWidth;

    /* Expander size */
    gint expanderWidth;
    gint expanderHeight;

    /* Menu separator height; */
    gint menuSeparatorHeight;

    /* Splitter size */
    gint splitterHorizontalHeight;
    gint splitterVerticalWidth;

    /* Notebook tab thickness */
    gint notebookTabThickness;

    /* Images in menus? */
    gboolean menuImages;
} WebKitIRenderThemeWidgetMetrics;

/**
 * WebKitIRenderThemeScrollbarMetrics:
 * @slider_width: Description
 * @trough_border: Description
 * @stepper_size: Description
 * @stepper_spacing: Description
 * @min_slider_size: Description
 * @trough_under_steppers: Description
 * @has_secondary_forward_stepper: Description
 * @has_secondary_backward_stepper: Description
 * @has_buttons: Description
 **/
typedef struct {
  gint slider_width;
  gint trough_border;
  gint stepper_size;
  gint stepper_spacing;
  gint min_slider_size;
  gboolean trough_under_steppers;
  gboolean has_secondary_forward_stepper;
  gboolean has_secondary_backward_stepper;
  gboolean has_buttons;
} WebKitIRenderThemeScrollbarMetrics;

struct _WebKitIRenderThemeInterface {
    GTypeInterface parent;

    guint32 (*get_platform_color)       (WebKitIRenderTheme*,
                                         WebKitIRenderThemePlatformColor);
    guint32 (*get_system_color)         (WebKitIRenderTheme*,
                                         int);

    gboolean (*get_widget_metrics)   (WebKitIRenderTheme*,
                                      WebKitIRenderThemeWidgetMetrics*);
    gboolean (*get_scrollbar_metrics)   (WebKitIRenderTheme*,
                                         WebKitIRenderThemeScrollbarMetrics*);

    gboolean (*paint_widget)            (WebKitIRenderTheme*,
                                         WebKitIRenderThemeWidgetType,
                                         WebKitIRenderThemeWidgetState*,
                                         gint flags,
                                         cairo_t*,
                                         cairo_rectangle_int_t*);

    /* Padding for future expansion */
    void (*_webkit_reserved0) (void);
    void (*_webkit_reserved1) (void);
    void (*_webkit_reserved2) (void);
    void (*_webkit_reserved3) (void);
};

WEBKIT_API GType
webkit_irender_theme_get_type                        (void);

WEBKIT_API guint32
webkit_irender_theme_get_platform_color              (WebKitIRenderTheme*,
                                                      WebKitIRenderThemePlatformColor);
WEBKIT_API guint32
webkit_irender_theme_get_system_color                (WebKitIRenderTheme*,
                                                      int);

WEBKIT_API gboolean
webkit_irender_theme_get_widget_metrics              (WebKitIRenderTheme*,
                                                      WebKitIRenderThemeWidgetMetrics*);
WEBKIT_API gboolean
webkit_irender_theme_get_scrollbar_metrics           (WebKitIRenderTheme*,
                                                      WebKitIRenderThemeScrollbarMetrics*);

WEBKIT_API gboolean
webkit_irender_theme_paint_widget                    (WebKitIRenderTheme*,
                                                      WebKitIRenderThemeWidgetType,
                                                      WebKitIRenderThemeWidgetState*,
                                                      gint flags,
                                                      cairo_t*,
                                                      cairo_rectangle_int_t*);

/* Default implementation. */
WEBKIT_API WebKitIRenderTheme*
webkit_irender_theme_get_default                     (void);
WEBKIT_API void
webkit_irender_theme_set_default                     (WebKitIRenderTheme*);

G_END_DECLS

#endif
