/*
 * Copyright (C) 2010 Collabora Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef webkitiwebview_h
#define webkitiwebview_h

#include "webkitdom.h"
#include "webkitenumtypes.h"
#include "webkitirendertheme.h"
#include "webkitipopupmenu.h"
#include "webkitclutterinclude.h"
#include <webkit/webkitdefines.h>
#include <webkit/webkitwebpage.h>

G_BEGIN_DECLS

#define WEBKIT_TYPE_IWEB_VIEW                (webkit_iweb_view_get_type())
#define WEBKIT_IWEB_VIEW(obj)                (G_TYPE_CHECK_INSTANCE_CAST((obj), WEBKIT_TYPE_IWEB_VIEW, WebKitIWebView))
#define WEBKIT_IS_IWEB_VIEW(obj)             (G_TYPE_CHECK_INSTANCE_TYPE((obj), WEBKIT_TYPE_IWEB_VIEW))
#define WEBKIT_IWEB_VIEW_GET_INTERFACE(inst) (G_TYPE_INSTANCE_GET_INTERFACE((inst),  WEBKIT_TYPE_IWEB_VIEW, WebKitIWebViewInterface))

struct _WebKitIWebViewInterface {
    GTypeInterface parent;

    WebKitWebPage* (* get_page)             (WebKitIWebView*);

    WebKitIRenderTheme* (*get_render_theme) (WebKitIWebView*);

    void (* invalidate)       (WebKitIWebView*);

    void (* invalidate_rect)  (WebKitIWebView*,
                               ClutterGeometry*);

    void (* get_window_rect)  (WebKitIWebView*,
                               ClutterGeometry*);

    void (* set_window_rect)  (WebKitIWebView*,
                               ClutterGeometry*);

    gboolean (* handle_fullscreen_media)  (WebKitIWebView*,
                                           WebKitDOMHTMLVideoElement*,
                                           ClutterActor*);

    gboolean (* handle_exit_fullscreen_media) (WebKitIWebView*,
                                               ClutterActor*);

    void (* delegated_scroll) (WebKitIWebView*, int, int);

    WebKitIPopupMenu* (* create_popup_menu) (WebKitIWebView*);

    void (* add_tile)            (WebKitIWebView*,
                                  ClutterActor*);

    void (* add_plugin_actor)    (WebKitIWebView*,
                                  ClutterActor*);
    void (* remove_plugin_actor) (WebKitIWebView*,
                                  ClutterActor*);

    void (* post_commit_setup) (WebKitIWebView*,
                                gboolean);

    void (* cancel_gestures)     (WebKitIWebView*);

    /* Padding for future expansion */
    void (*_webkit_reserved0) (void);
    void (*_webkit_reserved1) (void);
    void (*_webkit_reserved2) (void);
    void (*_webkit_reserved3) (void);
};

WEBKIT_API GType
webkit_iweb_view_get_type                        (void);

WEBKIT_API void
webkit_iweb_view_load_uri                        (WebKitIWebView*,
                                                  const gchar*);

WEBKIT_API void
webkit_iweb_view_load_string                     (WebKitIWebView* view,
                                                  const gchar* content,
                                                  const gchar* mime_type,
                                                  const gchar* encoding,
                                                  const gchar* base_uri);

WEBKIT_API WebKitWebPage*
webkit_iweb_view_get_page                        (WebKitIWebView*);

WEBKIT_API WebKitIRenderTheme*
webkit_iweb_view_get_render_theme                (WebKitIWebView*);

WEBKIT_API void
webkit_iweb_view_invalidate                      (WebKitIWebView*);

WEBKIT_API void
webkit_iweb_view_invalidate_rect                 (WebKitIWebView*,
                                                  ClutterGeometry*);

WEBKIT_API void
webkit_iweb_view_get_window_rect                 (WebKitIWebView*,
                                                  ClutterGeometry*);
WEBKIT_API void
webkit_iweb_view_set_window_rect                 (WebKitIWebView*,
                                                  ClutterGeometry*);

WEBKIT_API WebKitIPopupMenu*
webkit_iweb_view_create_popup_menu               (WebKitIWebView*);

gboolean
webkit_iweb_view_handle_fullscreen_media         (WebKitIWebView*,
                                                  WebKitDOMHTMLVideoElement*,
                                                  ClutterActor*);

gboolean
webkit_iweb_view_handle_exit_fullscreen_media    (WebKitIWebView* view,
                                                  ClutterActor* texture);

WEBKIT_API void
webkit_iweb_view_add_plugin_actor                (WebKitIWebView*,
                                                  ClutterActor*);
WEBKIT_API void
webkit_iweb_view_remove_plugin_actor             (WebKitIWebView*,
                                                  ClutterActor*);

void
webkit_iweb_view_post_commit_setup               (WebKitIWebView*,
                                                  gboolean resetScrolling);
void
webkit_iweb_view_add_tile                        (WebKitIWebView* view,
                                                  ClutterActor* actor);

void
webkit_iweb_view_delegated_scroll                (WebKitIWebView* view,
                                                  int x, int y);

WEBKIT_API void
webkit_iweb_view_cancel_gestures                 (WebKitIWebView* view);

WEBKIT_API WebKitIPopupMenu*
webkit_iweb_view_create_popup_menu               (WebKitIWebView*);

/* Default method implementations, to be used by any toolkit library. */
WEBKIT_API gboolean
webkit_iweb_view_button_press_event              (ClutterActor*,
                                                  ClutterButtonEvent*);
WEBKIT_API gboolean
webkit_iweb_view_button_release_event            (ClutterActor*,
                                                  ClutterButtonEvent*);
WEBKIT_API gboolean
webkit_iweb_view_motion_event                    (ClutterActor*,
                                                  ClutterMotionEvent* event);

WEBKIT_API gboolean
webkit_iweb_view_scroll_event                    (ClutterActor*,
                                                  ClutterScrollEvent* event);

WEBKIT_API gboolean
webkit_iweb_view_key_event                       (ClutterActor*,
                                                  ClutterKeyEvent* event);

WEBKIT_API gboolean
webkit_iweb_view_touch_event                     (ClutterActor*,
                                                  ClutterTouchEvent* event);

WEBKIT_API gboolean
webkit_iweb_view_gesture_scroll_begin            (ClutterActor*,
                                                  gint64              timestamp,
                                                  gfloat              event_x,
                                                  gfloat              event_y);

WEBKIT_API gboolean
webkit_iweb_view_gesture_scroll_update           (ClutterActor*,
                                                  gint64              timestamp,
                                                  gfloat              event_x,
                                                  gfloat              event_y,
                                                  gfloat              delta_x,
                                                  gfloat              delta_y);

WEBKIT_API gboolean
webkit_iweb_view_gesture_scroll_end              (ClutterActor*,
                                                  gint64              timestamp,
                                                  gfloat              event_x,
                                                  gfloat              event_y);

WEBKIT_API gboolean
webkit_iweb_view_gesture_scroll_momentum_begin   (ClutterActor*,
                                                  gint64              timestamp,
                                                  gfloat              event_x,
                                                  gfloat              event_y);

WEBKIT_API gboolean
webkit_iweb_view_gesture_scroll_momentum_update  (ClutterActor*,
                                                  gint64              timestamp,
                                                  gfloat              event_x,
                                                  gfloat              event_y,
                                                  gfloat              delta_x,
                                                  gfloat              delta_y);

WEBKIT_API gboolean
webkit_iweb_view_gesture_scroll_momentum_end     (ClutterActor*,
                                                  gint64              timestamp,
                                                  gfloat              event_x,
                                                  gfloat              event_y);

WEBKIT_API gboolean
webkit_iweb_view_gesture_tap_begin               (ClutterActor*,
                                                  gint64              timestamp,
                                                  gfloat              event_x,
                                                  gfloat              event_y,
                                                  ClutterModifierType state);

WEBKIT_API gboolean
webkit_iweb_view_gesture_tap                     (ClutterActor*,
                                                  gint64              timestamp,
                                                  gfloat              event_x,
                                                  gfloat              event_y,
                                                  ClutterModifierType state);

WEBKIT_API gboolean
webkit_iweb_view_gesture_tap_cancel              (ClutterActor*,
                                                  gint64              timestamp,
                                                  gfloat              event_x,
                                                  gfloat              event_y,
                                                  ClutterModifierType state);

WEBKIT_API void
webkit_iweb_view_key_focus_in                    (ClutterActor*);

WEBKIT_API void
webkit_iweb_view_key_focus_out                   (ClutterActor*);

WEBKIT_API void
webkit_iweb_view_pop_gl_context                  (ClutterActor*);

G_END_DECLS

#endif
