/*
 * Copyright (C) 2010 Collabora Ltd.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef webkitwebpage_h
#define webkitwebpage_h

#include <webkit/webkitdefines.h>
#include <webkit/webkitdom.h>
#include "webkitenumtypes.h"

G_BEGIN_DECLS

#define WEBKIT_TYPE_WEB_PAGE            (webkit_web_page_get_type())
#define WEBKIT_WEB_PAGE(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), WEBKIT_TYPE_WEB_PAGE, WebKitWebPage))
#define WEBKIT_WEB_PAGE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass),  WEBKIT_TYPE_WEB_PAGE, WebKitWebPageClass))
#define WEBKIT_IS_WEB_PAGE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), WEBKIT_TYPE_WEB_PAGE))
#define WEBKIT_IS_WEB_PAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),  WEBKIT_TYPE_WEB_PAGE))
#define WEBKIT_WEB_PAGE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj),  WEBKIT_TYPE_WEB_PAGE, WebKitWebPageClass))

typedef struct _AtkObject AtkObject;
typedef struct _ClutterActor ClutterActor;
typedef struct _WebKitWebPagePrivate WebKitWebPagePrivate;

/**
 * WebKitWebPagePageMode:
 * @WEBKIT_WEB_PAGE_PAGE_MODE_WINDOWED: description
 * @WEBKIT_WEB_PAGE_PAGE_MODE_FLOATING: description
 * @WEBKIT_WEB_PAGE_PAGE_MODE_FULLSCREEN: description
 * @WEBKIT_WEB_PAGE_PAGE_MODE_MAXIMIZED: description
 * @WEBKIT_WEB_PAGE_PAGE_MODE_MINIMIZED: description
 *
 * Update dscription.
 **/
typedef enum
{
    WEBKIT_WEB_PAGE_PAGE_MODE_WINDOWED,
    WEBKIT_WEB_PAGE_PAGE_MODE_FLOATING,
    WEBKIT_WEB_PAGE_PAGE_MODE_FULLSCREEN,
    WEBKIT_WEB_PAGE_PAGE_MODE_MAXIMIZED,
    WEBKIT_WEB_PAGE_PAGE_MODE_MINIMIZED
} WebKitWebPagePageMode;

struct _WebKitWebPage {
    GObject parent_instance;

    /*< private >*/
    WebKitWebPagePrivate *priv;
};

struct _WebKitWebPageClass {
    GObjectClass parent_class;

    /* Padding for future expansion */
    void (*_webkit_reserved0) (void);
    void (*_webkit_reserved1) (void);
    void (*_webkit_reserved2) (void);
    void (*_webkit_reserved3) (void);
};

/**
 * ChooseElementToZoomCallback:
 * @candidates: a list of #WebKitDOMElement
 * @userData: a data pointer
 *
 * The type of functions which are used to select the most relevant
 * node when operating a contextual zoom.
 *
 * Returns: the node to zoom to.
 */
typedef WebKitDOMElement* (*ChooseElementToZoomCallback) (GSList* candidates, void* userData);

WEBKIT_API GType
webkit_web_page_get_type                        (void);

WEBKIT_API WebKitWebPage*
webkit_web_page_new                             (WebKitIWebView* web_page);

WEBKIT_API WebKitIWebView*
webkit_web_page_get_view                        (WebKitWebPage*);

WEBKIT_API WebKitWebFrame*
webkit_web_page_get_main_frame                  (WebKitWebPage* web_page);

WEBKIT_API void
webkit_web_page_get_viewport_size               (WebKitWebPage* webPage,
                                                 int* width, int* height);

WEBKIT_API void
webkit_web_page_set_viewport_size               (WebKitWebPage* webPage,
                                                 const int width,
                                                 const int height);

WEBKIT_API void
webkit_web_page_reload                          (WebKitWebPage* web_page);

WEBKIT_API void
webkit_web_page_reload_bypass_cache             (WebKitWebPage* web_page);

WEBKIT_API void
webkit_web_page_load_uri                        (WebKitWebPage* web_page,
                                                 const gchar* uri);

WEBKIT_API void
webkit_web_page_load_string                     (WebKitWebPage* webPage,
                                                 const gchar* content,
                                                 const gchar* mimeType,
                                                 const gchar* encoding,
                                                 const gchar* baseUri);

WEBKIT_API void
webkit_web_page_load_request                    (WebKitWebPage* web_page,
                                                 WebKitNetworkRequest* request);

WEBKIT_API gboolean
webkit_web_page_search_text                     (WebKitWebPage* web_page,
                                                 const gchar* text,
                                                 gboolean case_sensitive,
                                                 gboolean forward,
                                                 gboolean wrap);

WEBKIT_API guint
webkit_web_page_mark_text_matches               (WebKitWebPage* web_page,
                                                 const gchar* string,
                                                 gboolean case_sensitive,
                                                 guint limit);

WEBKIT_API void
webkit_web_page_set_highlight_text_matches      (WebKitWebPage* web_page,
                                                 gboolean highlight);

WEBKIT_API void
webkit_web_page_unmark_text_matches             (WebKitWebPage* web_page);

WEBKIT_API void
webkit_web_page_set_settings                    (WebKitWebPage* web_page,
                                                 WebKitWebSettings* settings);

WEBKIT_API WebKitWebSettings*
webkit_web_page_get_settings                    (WebKitWebPage* web_page);

WEBKIT_API WebKitWebInspector*
webkit_web_page_get_inspector                   (WebKitWebPage* web_page);

WEBKIT_API WebKitLoadStatus
webkit_web_page_get_load_status                 (WebKitWebPage* web_page);

WEBKIT_API gdouble
webkit_web_page_get_progress                    (WebKitWebPage* web_page);

WEBKIT_API gboolean
webkit_web_page_can_go_back                     (WebKitWebPage* web_page);

WEBKIT_API gboolean
webkit_web_page_can_go_back_or_forward          (WebKitWebPage* web_page,
                                                 gint steps);

WEBKIT_API gboolean
webkit_web_page_can_go_forward                  (WebKitWebPage* web_page);

WEBKIT_API void
webkit_web_page_go_back                         (WebKitWebPage* web_page);

WEBKIT_API void
webkit_web_page_go_back_or_forward              (WebKitWebPage* web_page,
                                                 gint steps);

WEBKIT_API void
webkit_web_page_go_forward                      (WebKitWebPage* web_page);

WEBKIT_API void
webkit_web_page_set_maintains_back_forward_list (WebKitWebPage* web_page,
                                                 gboolean flag);

WEBKIT_API WebKitWebBackForwardList*
webkit_web_page_get_back_forward_list           (WebKitWebPage* web_page);

WEBKIT_API gboolean
webkit_web_page_go_to_back_forward_item         (WebKitWebPage* web_page,
                                                 WebKitWebHistoryItem* item);

WEBKIT_API AtkObject*
webkit_web_page_get_accessible                  (WebKitWebPage* page);

WEBKIT_API void
webkit_web_page_stop_loading                    (WebKitWebPage* web_page);

WEBKIT_API const gchar *
webkit_web_page_get_title                       (WebKitWebPage* web_page);

WEBKIT_API const char*
webkit_web_page_get_uri                         (WebKitWebPage* web_page);

WEBKIT_API WebKitWebFrame*
webkit_web_page_get_main_frame                  (WebKitWebPage*);

WEBKIT_API WebKitWebWindowFeatures*
webkit_web_page_get_window_features             (WebKitWebPage* webPage);

WEBKIT_API WebKitDOMDocument *
webkit_web_page_get_dom_document                (WebKitWebPage* webPage);

WEBKIT_API WebKitHitTestResult*
webkit_web_page_hit_test                        (WebKitWebPage* webPage,
                                                 int x,
                                                 int y);


WEBKIT_API WebKitHitTestResult*
webkit_web_page_hit_test_rectangle              (WebKitWebPage* webPage,
                                                 int x, int y,
                                                 guint topPadding,
                                                 guint bottomPadding,
                                                 guint leftPadding,
                                                 guint rightPadding);

WEBKIT_API gboolean
webkit_web_page_get_full_content_zoom           (WebKitWebPage* web_page);

WEBKIT_API void
webkit_web_page_set_full_content_zoom           (WebKitWebPage* web_page,
                                                 gboolean full_content_zoom);

WEBKIT_API gfloat
webkit_web_page_get_zoom_level                  (WebKitWebPage* web_page);

WEBKIT_API void
webkit_web_page_set_zoom_level                  (WebKitWebPage* web_page,
                                                 gfloat zoom_level);

WEBKIT_API void
webkit_web_page_zoom_in                         (WebKitWebPage* web_page);

WEBKIT_API void
webkit_web_page_zoom_out                        (WebKitWebPage* web_page);

WEBKIT_API WebKitWebFrame*
webkit_web_page_get_focused_frame               (WebKitWebPage* webPage);

WEBKIT_API void
webkit_web_page_execute_script                  (WebKitWebPage* webPage,
                                                 const gchar* script);

WEBKIT_API const gchar *
webkit_web_page_get_icon_uri                    (WebKitWebPage* web_page);

WEBKIT_API gboolean
webkit_web_page_get_editable                    (WebKitWebPage* web_page);

WEBKIT_API void
webkit_web_page_set_editable                    (WebKitWebPage* web_page,
                                                 gboolean flag);

WEBKIT_API gboolean
webkit_web_page_can_show_mime_type              (WebKitWebPage* web_page,
                                                 const gchar* mime_type);


WEBKIT_API const gchar*
webkit_web_page_get_encoding                    (WebKitWebPage* webPage);

WEBKIT_API void
webkit_web_page_set_custom_encoding             (WebKitWebPage* webPage,
                                                 const gchar* encoding);

WEBKIT_API const char*
webkit_web_page_get_custom_encoding             (WebKitWebPage* webPage);

WEBKIT_API void
webkit_web_page_set_page_mode                   (WebKitWebPage* web_page,
                                                 WebKitWebPagePageMode mode);

WEBKIT_API WebKitWebPagePageMode
webkit_web_page_get_page_mode                   (WebKitWebPage* web_page);

WEBKIT_API void
webkit_web_page_load_image                      (WebKitWebPage* webPage,
                                                 const gchar *url);

WEBKIT_API gboolean
webkit_web_page_get_use_fixed_layout            (WebKitWebPage* webPage);

WEBKIT_API void
webkit_web_page_set_use_fixed_layout            (WebKitWebPage* webPage,
                                                 gboolean used);

WEBKIT_API guint
webkit_web_page_get_fixed_layout_width          (WebKitWebPage* webPage);

WEBKIT_API void
webkit_web_page_set_fixed_layout_width          (WebKitWebPage* webPage,
                                                 guint width);

WEBKIT_API guint
webkit_web_page_get_fixed_layout_height         (WebKitWebPage* webPage);

WEBKIT_API void
webkit_web_page_set_fixed_layout_height         (WebKitWebPage* webPage,
                                                 guint height);

WEBKIT_API bool
webkit_web_page_get_zoom_target                 (WebKitWebPage* webPage,
                                                 float x,
                                                 float y,
                                                 int *zoomCenterX,
                                                 int *zoomCenterY,
                                                 double *zoomFactor,
                                                 ChooseElementToZoomCallback callback,
                                                 void* userData);
G_END_DECLS

#endif
